==> superd-0.2.tar.gz: superd-0.2/internal/config/parse_test.go <==
// Copyright 2022 Clayton Craft <clayton@craftyguy.net>
// SPDX-License-Identifier: GPL-3.0-or-later

package config

import (
	"bytes"
	"errors"
	"reflect"
	"testing"
	"time"
)

func TestReplaceInstanceName(t *testing.T) {
	subtests := []struct {
		name     string
		inBytes  []byte
		inName   string
		expected []byte
	}{
		{
			name:     "only 1 %i",
			inBytes:  []byte{69, 120, 101, 99, 61, 98, 97, 114, 32, 45, 99, 32, 37, 105, 32, 45, 118, 32, 45, 45, 119, 104, 97, 116, 101, 118, 101, 114},
			inName:   "fooooooooo",
			expected: []byte{69, 120, 101, 99, 61, 98, 97, 114, 32, 45, 99, 32, 102, 111, 111, 111, 111, 111, 111, 111, 111, 111, 32, 45, 118, 32, 45, 45, 119, 104, 97, 116, 101, 118, 101, 114},
		},
		{
			name:     "no %i",
			inBytes:  []byte{37, 32, 105, 32, 37, 98, 32, 105, 37},
			inName:   "pretzels",
			expected: []byte{37, 32, 105, 32, 37, 98, 32, 105, 37},
		},
		{
			name:     "3 %i's",
			inBytes:  []byte{69, 120, 101, 99, 83, 116, 97, 114, 116, 61, 100, 111, 115, 116, 117, 102, 102, 32, 37, 105, 32, 45, 99, 32, 37, 105, 32, 37, 105, 32, 37, 106, 32, 37, 107, 32, 37, 108},
			inName:   "twix",
			expected: []byte{69, 120, 101, 99, 83, 116, 97, 114, 116, 61, 100, 111, 115, 116, 117, 102, 102, 32, 116, 119, 105, 120, 32, 45, 99, 32, 116, 119, 105, 120, 32, 116, 119, 105, 120, 32, 37, 106, 32, 37, 107, 32, 37, 108},
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			out := replaceInstanceName(subtest.inBytes, subtest.inName)
			if !bytes.Equal(out, subtest.expected) {
				t.Fatalf("Expected: %q, got: %q", subtest.expected, out)
			}
		})
	}
}

func TestInstanceName(t *testing.T) {
	subtests := []struct {
		name     string
		in       string
		expected string
	}{
		{
			name:     "basic instance name",
			in:       "/path/to/some/foo@bar.service",
			expected: "bar",
		},
		{
			name:     "template",
			in:       "/etc/services/@bar.service",
			expected: "",
		},
		{
			name:     "valid instance with deep path",
			in:       "/////////////foo@bar.service",
			expected: "bar",
		},
		{
			name:     "invalid template/instance name",
			in:       "/path/to/some/foo@.service",
			expected: "",
		},
		{
			name:     "non-template config file",
			in:       "/path/to/some/foo.service",
			expected: "",
		},
		{
			name:     "not a valid anything",
			in:       "apples?!",
			expected: "",
		},
		{
			name:     "file with many @s",
			in:       "foo@bar@bazz.service",
			expected: "",
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			out := instanceName(subtest.in)
			if out != subtest.expected {
				t.Fatalf("Expected: %q, got: %q", subtest.expected, out)
			}
		})
	}
}

func TestParseConfig(t *testing.T) {
	subtests := []struct {
		name        string
		in          systemdConfig
		expected    *ServiceConfig
		expectedErr error
	}{
		{
			name: "valid config",
			in: systemdConfig{
				Unit: unit{
					Description: "this is a test of a valid .service config file",
					After:       "bazz boop",
					Before:      "bar",
				},
				Service: service{
					Type:          "Simple",
					ExecStart:     "/usr/bin/foo --bar --bazz 127",
					ExecStartPre:  "",
					ExecStartPost: "/bin/true",
					ExecOnFailure: "/bin/alert -b everybody_panic",
					Restart:       "Always",
					RestartSec:    "4",
				},
			},
			expected: &ServiceConfig{
				Name:                 "validconfigtest",
				Type:                 TypeSimple,
				Restart:              RestartAlways,
				RestartDur:           4 * time.Second,
				Cmd:                  "/usr/bin/foo",
				CmdArgs:              []string{"--bar", "--bazz", "127"},
				ExecStartPostCmd:     "/bin/true",
				ExecOnFailureCmd:     "/bin/alert",
				ExecOnFailureCmdArgs: []string{"-b", "everybody_panic"},
				Before:               []string{"bar"},
				After:                []string{"bazz", "boop"},
			},
			expectedErr: nil,
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			out, err := parseConfig(subtest.expected.Name, &subtest.in)
			if err != nil {
				if !errors.Is(err, subtest.expectedErr) {
					t.Fatalf("unexpected error: %s", err)
				}
				// this was the test, congrats you passed
				return
			}
			if !reflect.DeepEqual(out, subtest.expected) {
				t.Fatalf("Expected: %v, got: %v", subtest.expected, out)
			}
		})
	}
}

func TestParseRestartDur(t *testing.T) {
	subtests := []struct {
		name string
		in   string
		// string to feed directly to time.ParseDuration
		expected    string
		expectedErr bool
	}{
		{
			name:     "int value as seconds",
			in:       "9000",
			expected: "9000s",
		},
		{
			name:     "time span in systemd.time format",
			in:       "6h 20m 11s",
			expected: "6h20m11s",
		},
		{
			name:     "valid but backwards",
			in:       "1s 10m",
			expected: "10m1s",
		},
		{
			name:        "invalid value",
			in:          "banananananananananana",
			expectedErr: true,
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			out, err := parseRestartDur(subtest.in)
			if err != nil {
				if !subtest.expectedErr {
					t.Fatalf("unexpected error: %s", err)
				} else {
					switch err.(type) {
					case ErrUnsupportedValue:
						// this was the test, congrats you passed
						return
					default:
						t.Fatalf("unexpected error: %s", err)
					}
				}
			} else if subtest.expectedErr {
				t.Fatalf("expected parsing error but did not receive one")
			}

			e, err := time.ParseDuration(subtest.expected)
			if err != nil {
				t.Fatalf("unable to parse expected time duration of %q: %s", subtest.expected, err)
			}

			if out != e {
				t.Fatalf("expected: %q, got: %q", e, out)
			}
		})
	}
}

func TestParseCmd(t *testing.T) {
	subtests := []struct {
		name         string
		in           string
		inAllowEmpty bool
		expectedCmd  string
		expectedArgs []string
		expectedErr  bool
	}{
		{
			name:         "valid command + args",
			in:           "/usr/bin/flup -a -T 32",
			expectedCmd:  "/usr/bin/flup",
			expectedArgs: []string{"-a", "-T", "32"},
		},
		{
			name:        "valid command + no args",
			in:          "sway",
			expectedCmd: "sway",
		},
		{
			name:        "empty and allowEmpty=false",
			in:          "",
			expectedErr: true,
		},
		{
			name:         "empty and allowEmpty=true",
			in:           "",
			inAllowEmpty: true,
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			outCmd, outArgs, err := parseCmd(subtest.in, subtest.inAllowEmpty)
			if err != nil {
				if !subtest.expectedErr {
					t.Fatalf("unexpected error: %s", err)
				} else {
					switch err.(type) {
					case ErrUnsupportedValue:
						// this was the test, congrats you passed
						return
					default:
						t.Fatalf("unexpected error: %s", err)
					}
				}
			} else if subtest.expectedErr {
				t.Fatalf("expected parsing error but did not receive one")
			}

			if outCmd != subtest.expectedCmd {
				t.Fatalf("expected: %q, got: %q", subtest.expectedCmd, outCmd)
			}

			if !reflect.DeepEqual(outArgs, subtest.expectedArgs) {
				t.Fatalf("expected: %v, got: %v", subtest.expectedArgs, outArgs)
			}
		})
	}
}

func TestParseType(t *testing.T) {
	subtests := []struct {
		name        string
		in          string
		expected    Type
		expectedErr bool
	}{
		{
			name:     "simple type, lower case",
			in:       "Simple",
			expected: TypeSimple,
		},
		{
			name:     "oneshot type",
			in:       "oNeShOT",
			expected: TypeOneshot,
		},
		{
			name:        "unsupported type",
			in:          "guava",
			expectedErr: true,
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			out, err := parseType(subtest.in)
			if err != nil {
				if !subtest.expectedErr {
					t.Fatalf("unexpected error: %s", err)
				} else {
					switch err.(type) {
					case ErrUnsupportedValue:
						// this was the test, congrats you passed
						return
					default:
						t.Fatalf("unexpected error: %s", err)
					}
				}
			} else if subtest.expectedErr {
				t.Fatalf("expected parsing error but did not receive one")
			}

			if out != subtest.expected {
				t.Fatalf("expected: %v, got: %v", subtest.expected, out)
			}
		})
	}
}

func TestParseRestart(t *testing.T) {
	subtests := []struct {
		name        string
		in          string
		expected    RestartCond
		expectedErr bool
	}{
		{
			name:     "always, lower case",
			in:       "always",
			expected: RestartAlways,
		},
		{
			name:     "always, crazy case",
			in:       "AlWaYs",
			expected: RestartAlways,
		},
		{
			name:     "never",
			in:       "nO",
			expected: RestartNever,
		},
		{
			name:     "failure",
			in:       "on-failure",
			expected: RestartFailure,
		},
		{
			name:     "success",
			in:       "on-Success",
			expected: RestartSuccess,
		},
		{
			name:        "unsupported restart condition",
			in:          "dragon fruit",
			expectedErr: true,
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			out, err := parseRestart(subtest.in)
			if err != nil {
				if !subtest.expectedErr {
					t.Fatalf("unexpected error: %s", err)
				} else {
					switch err.(type) {
					case ErrUnsupportedValue:
						// this was the test, congrats you passed
						return
					default:
						t.Fatalf("unexpected error: %s", err)
					}
				}
			} else if subtest.expectedErr {
				t.Fatalf("expected parsing error but did not receive one")
			}

			if out != subtest.expected {
				t.Fatalf("expected: %v, got: %v", subtest.expected, out)
			}
		})
	}
}

func TestParseDeps(t *testing.T) {
	subtests := []struct {
		name        string
		in          string
		expected    []string
		expectedErr bool
	}{
		{
			name:     "basic no tricks",
			in:       "foo bar bazz",
			expected: []string{"foo", "bar", "bazz"},
		},
		{
			name: "empty",
			in:   "",
		},
		{
			name:     "with .service ext",
			in:       "foo.service bar bazz.SERVICE",
			expected: []string{"foo", "bar", "bazz"},
		},
		{
			name:        "with unknown ext",
			in:          "foo bar.whatever bazz",
			expectedErr: true,
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			out, err := parseDeps(subtest.in)
			if err != nil {
				if !subtest.expectedErr {
					t.Fatalf("unexpected error: %s", err)
				} else {
					switch err.(type) {
					case ErrUnsupportedValue:
						// this was the test, congrats you passed
						return
					default:
						t.Fatalf("unexpected error: %s", err)
					}
				}
			} else if subtest.expectedErr {
				t.Fatalf("expected parsing error but did not receive one")
			}

			if !reflect.DeepEqual(out, subtest.expected) {
				t.Fatalf("expected: %v, got: %v", subtest.expected, out)
			}
		})
	}
}

==> superd-0.3.2.tar.gz: superd-0.3.2/internal/config/parse_test.go <==
// Copyright 2022 Clayton Craft <clayton@craftyguy.net>
// SPDX-License-Identifier: GPL-3.0-or-later

package config

import (
	"bytes"
	"encoding/json"
	"errors"
	"reflect"
	"testing"
	"time"
)

func TestReplaceInstanceName(t *testing.T) {
	subtests := []struct {
		name     string
		inBytes  []byte
		inName   string
		expected []byte
	}{
		{
			name:     "only 1 %i",
			inBytes:  []byte{69, 120, 101, 99, 61, 98, 97, 114, 32, 45, 99, 32, 37, 105, 32, 45, 118, 32, 45, 45, 119, 104, 97, 116, 101, 118, 101, 114},
			inName:   "fooooooooo",
			expected: []byte{69, 120, 101, 99, 61, 98, 97, 114, 32, 45, 99, 32, 102, 111, 111, 111, 111, 111, 111, 111, 111, 111, 32, 45, 118, 32, 45, 45, 119, 104, 97, 116, 101, 118, 101, 114},
		},
		{
			name:     "no %i",
			inBytes:  []byte{37, 32, 105, 32, 37, 98, 32, 105, 37},
			inName:   "pretzels",
			expected: []byte{37, 32, 105, 32, 37, 98, 32, 105, 37},
		},
		{
			name:     "3 %i's",
			inBytes:  []byte{69, 120, 101, 99, 83, 116, 97, 114, 116, 61, 100, 111, 115, 116, 117, 102, 102, 32, 37, 105, 32, 45, 99, 32, 37, 105, 32, 37, 105, 32, 37, 106, 32, 37, 107, 32, 37, 108},
			inName:   "twix",
			expected: []byte{69, 120, 101, 99, 83, 116, 97, 114, 116, 61, 100, 111, 115, 116, 117, 102, 102, 32, 116, 119, 105, 120, 32, 45, 99, 32, 116, 119, 105, 120, 32, 116, 119, 105, 120, 32, 37, 106, 32, 37, 107, 32, 37, 108},
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			out := replaceInstanceName(subtest.inBytes, subtest.inName)
			if !bytes.Equal(out, subtest.expected) {
				t.Fatalf("Expected: %q, got: %q", subtest.expected, out)
			}
		})
	}
}

func TestInstanceName(t *testing.T) {
	subtests := []struct {
		name     string
		in       string
		expected string
	}{
		{
			name:     "basic instance name",
			in:       "/path/to/some/foo@bar.service",
			expected: "bar",
		},
		{
			name:     "template",
			in:       "/etc/services/@bar.service",
			expected: "",
		},
		{
			name:     "valid instance with deep path",
			in:       "/////////////foo@bar.service",
			expected: "bar",
		},
		{
			name:     "invalid template/instance name",
			in:       "/path/to/some/foo@.service",
			expected: "",
		},
		{
			name:     "non-template config file",
			in:       "/path/to/some/foo.service",
			expected: "",
		},
		{
			name:     "not a valid anything",
			in:       "apples?!",
			expected: "",
		},
		{
			name:     "file with many @s",
			in:       "foo@bar@bazz.service",
			expected: "",
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			out := instanceName(subtest.in)
			if out != subtest.expected {
				t.Fatalf("Expected: %q, got: %q", subtest.expected, out)
			}
		})
	}
}

func TestParseConfig(t *testing.T) {
	subtests := []struct {
		name        string
		in          systemdConfig
		expected    *ServiceConfig
		expectedErr error
	}{
		{
			name: "valid config",
			in: systemdConfig{
				Unit: unit{
					Description:           "this is a test of a valid .service config file",
					After:                 "bazz boop",
					Before:                "bar",
					StartLimitIntervalSec: "3s",
					StartLimitBurst:       12,
				},
				Service: service{
					Type:          "Simple",
					ExecStart:     "/usr/bin/foo --bar --bazz 127",
					ExecStartPre:  "",
					ExecStartPost: "/bin/true",
					ExecOnFailure: "/bin/alert -b everybody_panic",
					Restart:       "Always",
					RestartSec:    "4",
				},
			},
			expected: &ServiceConfig{
				Name:                 "validconfigtest",
				Description:          "this is a test of a valid .service config file",
				Type:                 TypeSimple,
				Restart:              RestartAlways,
				RestartDur:           4 * time.Second,
				Cmd:                  "/usr/bin/foo",
				CmdArgs:              []string{"--bar", "--bazz", "127"},
				ExecStartPostCmd:     "/bin/true",
				ExecOnFailureCmd:     "/bin/alert",
				ExecOnFailureCmdArgs: []string{"-b", "everybody_panic"},
				Before:               []string{"bar"},
				After:                []string{"bazz", "boop"},
				StartLimitDur:        3 * time.Second,
				StartLimitBurst:      12,
			},
			expectedErr: nil,
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			out, err := parseConfig(subtest.expected.Name, &subtest.in)
			if err != nil {
				if !errors.Is(err, subtest.expectedErr) {
					t.Fatalf("unexpected error: %s", err)
				}
				// this was the test, congrats you passed
				return
			}
			if !reflect.DeepEqual(out, subtest.expected) {
				o, _ := json.MarshalIndent(out, "", "\t")
				e, _ := json.MarshalIndent(subtest.expected, "", "\t")
				t.Fatalf("\n**Expected: %s\n**Got: %s", string(o), string(e))
			}
		})
	}
}

func TestParseRestartDur(t *testing.T) {
	subtests := []struct {
		name string
		in   string
		// string to feed directly to time.ParseDuration
		expected    string
		expectedErr bool
	}{
		{
			name:     "int value as seconds",
			in:       "9000",
			expected: "9000s",
		},
		{
			name:     "time span in systemd.time format",
			in:       "6h 20m 11s",
			expected: "6h20m11s",
		},
		{
			name:     "valid but backwards",
			in:       "1s 10m",
			expected: "10m1s",
		},
		{
			name:        "invalid value",
			in:          "banananananananananana",
			expectedErr: true,
		},
		{
			name:     "int value as seconds, with newline",
			in:       "9000\r\n",
			expected: "9000s",
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			out, err := parseDur(subtest.in, "Restart=", false, defaultRestartDur)
			if err != nil {
				if !subtest.expectedErr {
					t.Fatalf("unexpected error: %s", err)
				} else {
					switch err.(type) {
					case ErrUnsupportedValue:
						// this was the test, congrats you passed
						return
					default:
						t.Fatalf("unexpected error: %s", err)
					}
				}
			} else if subtest.expectedErr {
				t.Fatalf("expected parsing error but did not receive one")
			}

			e, err := time.ParseDuration(subtest.expected)
			if err != nil {
				t.Fatalf("unable to parse expected time duration of %q: %s", subtest.expected, err)
			}

			if out != e {
				t.Fatalf("expected: %q, got: %q", e, out)
			}
		})
	}
}

func TestParseCmd(t *testing.T) {
	subtests := []struct {
		name         string
		in           string
		inAllowEmpty bool
		expectedCmd  string
		expectedArgs []string
		expectedErr  bool
	}{
		{
			name:         "valid command + args",
			in:           "/usr/bin/flup -a -T 32",
			expectedCmd:  "/usr/bin/flup",
			expectedArgs: []string{"-a", "-T", "32"},
		},
		{
			name:        "valid command + no args",
			in:          "sway",
			expectedCmd: "sway",
		},
		{
			name:        "empty and allowEmpty=false",
			in:          "",
			expectedErr: true,
		},
		{
			name:         "empty and allowEmpty=true",
			in:           "",
			inAllowEmpty: true,
		},
		{
			name:         "valid command, multiline",
			in:           "sleep \\ 123 \\ -h",
			expectedCmd:  "sleep",
			expectedArgs: []string{"123", "-h"},
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			outCmd, outArgs, err := parseCmd(subtest.in, subtest.inAllowEmpty)
			if err != nil {
				if !subtest.expectedErr {
					t.Fatalf("unexpected error: %s", err)
				} else {
					switch err.(type) {
					case ErrUnsupportedValue:
						// this was the test, congrats you passed
						return
					default:
						t.Fatalf("unexpected error: %s", err)
					}
				}
			} else if subtest.expectedErr {
				t.Fatalf("expected parsing error but did not receive one")
			}

			if outCmd != subtest.expectedCmd {
				t.Fatalf("expected: %q, got: %q", subtest.expectedCmd, outCmd)
			}

			if !reflect.DeepEqual(outArgs, subtest.expectedArgs) {
				t.Fatalf("expected: %v, got: %v", subtest.expectedArgs, outArgs)
			}
		})
	}
}

func TestParseType(t *testing.T) {
	subtests := []struct {
		name        string
		in          string
		expected    Type
		expectedErr bool
	}{
		{
			name:     "simple type, lower case",
			in:       "Simple",
			expected: TypeSimple,
		},
		{
			name:     "oneshot type",
			in:       "oNeShOT",
			expected: TypeOneshot,
		},
		{
			name:        "unsupported type",
			in:          "guava",
			expectedErr: true,
		},
		{
			name:     "simple type, with newline",
			in:       "simple\n",
			expected: TypeSimple,
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			out, err := parseType(subtest.in)
			if err != nil {
				if !subtest.expectedErr {
					t.Fatalf("unexpected error: %s", err)
				} else {
					switch err.(type) {
					case ErrUnsupportedValue:
						// this was the test, congrats you passed
						return
					default:
						t.Fatalf("unexpected error: %s", err)
					}
				}
			} else if subtest.expectedErr {
				t.Fatalf("expected parsing error but did not receive one")
			}

			if out != subtest.expected {
				t.Fatalf("expected: %v, got: %v", subtest.expected, out)
			}
		})
	}
}

func TestParseDur(t *testing.T) {
	subtests := []struct {
		name        string
		in          string
		inRequired  bool
		inDefault   time.Duration
		expected    time.Duration
		expectedErr error
	}{
		{
			name:     "non-empty",
			in:       "1337s",
			expected: 1337 * time.Second,
		},
		{
			name:        "required but empty",
			in:          "",
			inRequired:  true,
			expectedErr: ErrUnsupportedValue{},
		},
		{
			name:      "empty use default",
			in:        "",
			inDefault: 9000 * time.Second,
			expected:  9000 * time.Second,
		},
	}

	for _, st := range subtests {
		t.Run(st.name, func(t *testing.T) {
			if out, err := parseDur(st.in, "testoption=", st.inRequired, st.inDefault); err != nil {
				// note: TypeOf used instead of errors.Is since an instance of
				// the error is stored in the subtest (expectedErr), and I'm
				// unaware of any way to "store a type" for the comparison here
				if reflect.TypeOf(err) != reflect.TypeOf(st.expectedErr) {
					t.Fatalf("err expected: %T, err received: %T", st.expectedErr, err)
				}
			} else {
				if st.expectedErr != nil {
					t.Fatalf("err expected: %T, err received: %T", st.expectedErr, err)
				}
				if out != st.expected {
					t.Fatalf("expected: %v, got: %v", st.expected, out)

				}
			}
		})
	}
}

func TestParseRestart(t *testing.T) {
	subtests := []struct {
		name        string
		in          string
		expected    RestartCond
		expectedErr bool
	}{
		{
			name:     "always, lower case",
			in:       "always",
			expected: RestartAlways,
		},
		{
			name:     "always, crazy case",
			in:       "AlWaYs",
			expected: RestartAlways,
		},
		{
			name:     "never",
			in:       "nO",
			expected: RestartNever,
		},
		{
			name:     "failure",
			in:       "on-failure",
			expected: RestartFailure,
		},
		{
			name:     "success",
			in:       "on-Success",
			expected: RestartSuccess,
		},
		{
			name:        "unsupported restart condition",
			in:          "dragon fruit",
			expectedErr: true,
		},
		{
			name:     "success, with newline",
			in:       "on-success\n",
			expected: RestartSuccess,
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			out, err := parseRestart(subtest.in)
			if err != nil {
				if !subtest.expectedErr {
					t.Fatalf("unexpected error: %s", err)
				} else {
					switch err.(type) {
					case ErrUnsupportedValue:
						// this was the test, congrats you passed
						return
					default:
						t.Fatalf("unexpected error: %s", err)
					}
				}
			} else if subtest.expectedErr {
				t.Fatalf("expected parsing error but did not receive one")
			}

			if out != subtest.expected {
				t.Fatalf("expected: %v, got: %v", subtest.expected, out)
			}
		})
	}
}

func TestParseDeps(t *testing.T) {
	subtests := []struct {
		name        string
		in          string
		expected    []string
		expectedErr bool
	}{
		{
			name:     "basic no tricks",
			in:       "foo bar bazz",
			expected: []string{"foo", "bar", "bazz"},
		},
		{
			name: "empty",
			in:   "",
		},
		{
			name:     "with .service ext",
			in:       "foo.service bar bazz.SERVICE",
			expected: []string{"foo", "bar", "bazz"},
		},
		{
			name:        "with unknown ext",
			in:          "foo bar.whatever bazz",
			expectedErr: true,
		},
		{
			name:     "basic, with newline",
			in:       "foo bar bazz\n",
			expected: []string{"foo", "bar", "bazz"},
		},
	}

	for _, subtest := range subtests {
		t.Run(subtest.name, func(t *testing.T) {
			out, err := parseDeps(subtest.in)
			if err != nil {
				if !subtest.expectedErr {
					t.Fatalf("unexpected error: %s", err)
				} else {
					switch err.(type) {
					case ErrUnsupportedValue:
						// this was the test, congrats you passed
						return
					default:
						t.Fatalf("unexpected error: %s", err)
					}
				}
			} else if subtest.expectedErr {
				t.Fatalf("expected parsing error but did not receive one")
			}

			if !reflect.DeepEqual(out, subtest.expected) {
				t.Fatalf("expected: %v, got: %v", subtest.expected, out)
			}
		})
	}
}

func TestMultiline2Single(t *testing.T) {
	subtests := []struct {
		name     string
		in       string
		expected string
	}{
		{
			name:     "multiline with single backslash",
			in:       "this \\is a test\n",
			expected: "this is a test",
		},
		{
			name:     "single line",
			in:       "avocados are neat",
			expected: "avocados are neat",
		},
		{
			name:     "newline char",
			in:       "AllowMultilineValues enables\n a property",
			expected: "AllowMultilineValues enables a property",
		},
	}

	for _, st := range subtests {
		t.Run(st.name, func(t *testing.T) {
			out := multiline2single(st.in)
			if out != st.expected {
				t.Fatalf("expected: %v, got: %v", st.expected, out)
			}
		})
	}
}
